#!/usr/bin/env ruby

$strPath=File.split(`#{"which rqr.rb"}`.chomp)[0]+"/"

$strBinQr=$strPath+"rqr/qr"

require 'qte'
require 'qpe'

include Qte
include Qpe

$codec = QTextCodec.codecForName("Shift-JIS")

$chrZero = "0"[0]



class RqrEnv
  def initialize
  end
  def getDesktopWidth
    QApplication.desktop.width
  end
  def getDesktopHeight
    QApplication.desktop.height
  end
end

class RqrConf
  def initialize(strFile = "/home/zaurus/Settings/rqr.conf")
    @strConfFile = strFile
    if $env.getDesktopWidth == 640 then 
      @hDefaultConf = {
        "sizeFont" => "6",
        "sizeQr" => "0",
        "bMargin" => "TRUE",
        "pathCurPix" => "/home/zaurus/Documents/rqr.jpeg",
        "pathTmp" => "/tmp/qrinput.txt"
      }
    else
      @hDefaultConf = {
        "sizeFont" => "2",
        "sizeQr" => "0",
        "bMargin" => "TRUE",
        "pathCurPix" => "/home/zaurus/Documents/rqr.jpeg",
        "pathTmp" => "/tmp/qrinput.txt"
      }
    end
    if FileTest.directory?("/home/zaurus/")==false then
      @hDefaultConf["pathCurPix"] ="/home/root/Documents/rqr.jpeg"
    end
    @hConf=@hDefaultConf
    load
  end
  def load
    if FileTest.file?(@strConfFile)==false then
      save
      return @hConf
    end
    fp = File.new(@strConfFile,"r")
    fp.each do |line|
      line = line.chomp
      unless line =~ /^$/
        a = line.split("=",2)
        @hConf[a[0]]=a[1]
      end
    end
    @hDefaultConf.each do |k, v|
      unless @hConf.has_key?(k)
        @hConf[k]=v
      end
    end
    save
    return @hConf
  end
  def getConf
    @hConf
  end
  def setConf(h)
    @hConf=h
    save
  end
  def save
    fp = File.new(@strConfFile,"w")
    @hConf.each do |k, v|
      fp.print(k + "=" + v + "\n")
    end
    fp.close
  end
end

class RqrSetPixSizeDialog < QDialog
  def initialize
    super(nil, "RqrSetPixSizeDialog", true) # Modal Dialog
    setCaption(tr("サイズ設定"))
    vb=QVBox.new(self)
    if $env.getDesktopWidth == 640 then
      vb.setFixedSize(200,120)
    else
      vb.setFixedSize(100,100)
    end
      @ckAuto=QCheckBox.new(tr("自動調節"),vb)
      QLabel.new(tr("　チェックしない場合、"),vb)
      QLabel.new(tr("↓サイズを指定します。"),vb)
      hb=QHBox.new(vb)
        @sbSize=QSpinBox.new(hb)
        QLabel.new(tr("ピクセル"),hb)
        @sbSize.setMinValue(0)
        if $env.getDesktopWidth == 640 then
          @sbSize.setMaxValue(400)
        else
          @sbSize.setMaxValue(200)
        end
      @ckMargin=QCheckBox.new(tr("マージン"),vb)
      connect(@ckAuto,QSIGNAL("stateChanged(int)"),self,"tglCheck(i)")
  end
  def exec
    h=$conf.getConf
    if h["sizeQr"]=="0" then
      @ckAuto.setChecked(true)
      @sbSize.setValue(100)
      @sbSize.setEnabled(false)
    else
      @ckAuto.setChecked(false)
      @sbSize.setValue(h["sizeQr"].to_i)
      @sbSize.setEnabled(true)
    end
    if h["bMargin"]=="TRUE" then
      @ckMargin.setChecked(true)
    else
      @ckMargin.setChecked(false)
    end
    super
  end
  def getConf
    h=$conf.getConf
    if @ckAuto.isChecked then
      h["sizeQr"]="0"
    else
      h["sizeQr"]=@sbSize.value.to_s
    end
    if @ckMargin.isChecked then
      h["bMargin"]="TRUE"
    else
      h["bMargin"]="FALSE"
    end
    h
  end
  def tglCheck(i)
    case i
    when 0
      @sbSize.setEnabled(true)
    when 2
      @sbSize.setEnabled(false)
    end
  end
end

class RqrDialog < QDialog
  def initialize
    super(nil, "RqrDialog", true) # Modal Dialog
    if $env.getDesktopWidth == 640 then
      setFixedSize(400,400)
    else
      setFixedSize(200,200)
    end
    setCaption(tr("RQR"))
    @qrCanvas=RqrCanvas.new(self)
    @qrCanvas.setFixedSize(size)
  end
  def getCanvas
    @qrCanvas
  end
end

class RqrCanvas < QWidget
  def initialize(parent)
    super(parent)
    setPalette(QPalette.new(QColor.new(255,255,255)))
    @strQr=""
    @qpBuf=QPixmap.new(size)
    @qpBuf.fill(QColor.new(255,255,255))
    @p=QPainter.new
    catchEvent
  end
  def drawQr(strQr)
    @strQr=strQr
    if @strQr!="" then
      astrQr = @strQr.split(/\n/)
      h=$conf.getConf

      if h["bMargin"]=="TRUE" then
        strVMargin = "0" * (astrQr.length + 4)
        astrQr.collect! {|strOne|
          "00" + strOne + "00"
        }
        astrQr.unshift(strVMargin)
        astrQr.unshift(strVMargin)
        astrQr.push(strVMargin)
        astrQr.push(strVMargin)
      end

      widthQr = astrQr[0].length

      h=$conf.getConf

      if h["sizeQr"]=="0" then

        s=widthQr*$iBlock
        i=$iBlock
        j=$iBlock
        if s>height then
          i=height.to_f/widthQr.to_f
          j=i.ceil
          s=widthQr*i
        end
        if s>width then
          i=width.to_f/widthQr.to_f
          j=i.ceil
          s=widthQr*i
        end

      else

        s=h["sizeQr"].to_i
        i=s.to_f/widthQr.to_f
        j=i.ceil

      end

      @qpBuf.resize(s,s)

      @qpBuf.fill(QColor.new(255,255,255))
      @p.begin(@qpBuf)
      @p.setBrush(QColor.new(0,0,0))
      @p.setPen(SolidLine)

      y = 0
      while y < widthQr
        strOne = astrQr[y]
        x = 0
        while x < widthQr
          if strOne[x]!=$chrZero then
            @p.drawRect(x*i, y*i, j, j)
          end
          x += 1
        end
        y += 1
      end
      @p.end
      h=$conf.getConf
      @qpBuf.save(h["pathCurPix"],"JPEG")
    end
  end
  def paintEvent(e)
    bitBlt(self,width/2 - @qpBuf.width/2, height/2 - @qpBuf.height/2 ,@qpBuf)
  end
end

class RqrEdit < QMultiLineEdit
  def initialize(p)
    super(p)
    setWordWrap(WidgetWidth)
    setWrapPolicy(Anywhere)

    if FileTest.directory?("/home/zaurus/") then
      @aFontSize=[10,12,16,18,20,24,36,48,72]
    else
      @aFontSize=[10,12,16,18,20,24]
    end
    setupFontSize
    catchEvent
  end

  def setupFontSize
    hConf=$conf.getConf
    setFont(QFont.new("lcfont",@aFontSize[hConf["sizeFont"].to_i]))
  end

  def updownFont(i)
    hConf=$conf.getConf
    j=hConf["sizeFont"].to_i + i
    if j < 0 or j >= @aFontSize.size then
      return
    end
    hConf["sizeFont"]=j.to_s
    $conf.setConf(hConf)
    setupFontSize
  end

  def keyPressEvent(e)
    case e.key
    when Qt::Key_Tab
      $frmMain.execQr
      e.ignore
#    when 4154
#      $frmMain.setFocus2Menu
#      e.ignore
    end
  end
end

class RqrMailto < QVBox
  def initialize(p)
    super(p)
    h1=QHBox.new(self)
      QLabel.new(tr("宛先："),h1)
      @leTo = QLineEdit.new(h1)
    h2=QHBox.new(self)
      QLabel.new(tr("表題："),h2)
      @leSubject = QLineEdit.new(h2)
    @meBody = RqrEdit.new(self)
  end
  def setupFontSize
    @meBody.setupFontSize
  end

  def updownFont(i)
    @meBody.updownFont(i)
  end

  def replaceString(strIn)
    strIn.gsub(/\\/,"\\").gsub(/:/,"\:").gsub(/;/,"\;").gsub(/,/,"\,")
  end

  def getData
    strTo = replaceString($codec.fromUnicode(@leTo.text).data)
    strSubject = replaceString($codec.fromUnicode(@leSubject.text).data)
    strBody = replaceString($codec.fromUnicode(@meBody.text).data.gsub(/\n/,"\r\n"))
    "MATMSG:TO:" + strTo + ";SUB:" + strSubject + ";BODY:" + strBody + ";;"
  end

end

class FormMain < QMainWindow
  def initialize
    super
    setCaption("RQR")
    @idMail = 0
    @idText = 1
    vb=QVBox.new(self)
      @tab = QTabWidget.new(vb)
        @rqMail = RqrMailto.new(@tab)
        @tab.insertTab(@rqMail,tr("Mail"),@idMail)
        @leStr = RqrEdit.new(@tab)
        @tab.insertTab(@leStr,tr("Text"),@idText)
      @btnExec = QPushButton.new(tr("QRコードに変換"),vb)
    setCentralWidget(vb)
    @strLastData=""
    @strLastQr=""
    @strLastSize=""
    @strLastMargin=""
    @aiVMax = [-1,14,26,42,62,84,106,122,152,180,213,251,287,331,362,412,450,504,460,624,666,711,779,857,911,997,1059,1125,11901,1264,1370,1452,1538,1628,1722,1809,1911,1989,2099,2213,2331]
    @hMenuId=Hash.new
    @mbar=QMenuBar.new(self)
    @hPop=Hash.new
    ["ファイル","フォント"].each do |e|
      @hPop[e]=QPopupMenu.new
      @mbar.insertItem(tr(e),@hPop[e])
    end
    p=@hPop["ファイル"]
    @hMenuId["setPixPath"]=p.insertItem(tr("画像ファイル保存先変更"))
    @hMenuId["setPixSize"]=p.insertItem(tr("画像サイズ変更"))
    @hMenuId["execQr"]=p.insertItem(tr("QRコードに変換"))
    p=@hPop["フォント"]
    @hMenuId["upFontSize"]=p.insertItem(tr("もっと大きく"))
    @hMenuId["downFontSize"]=p.insertItem(tr("もっと小さく"))

    connect(@btnExec,QSIGNAL("clicked()"),self,"execQr")
    connect(@hPop["ファイル"],QSIGNAL("activated(int)"),self,"activateMenu(id)")
    connect(@hPop["フォント"],QSIGNAL("activated(int)"),self,"activateMenu(id)")
    catchEvent
  end

  def activateMenu(id)
    case id
    when @hMenuId["execQr"]
      execQr
    when @hMenuId["upFontSize"]
      @leStr.updownFont(1)
      @rqMail.setupFontSize
    when @hMenuId["downFontSize"]
      @leStr.updownFont(-1)
      @rqMail.setupFontSize
    when @hMenuId["setPixSize"]
      if $sizeDialog.exec !=0 then
        $conf.setConf($sizeDialog.getConf)
      end
    when @hMenuId["setPixPath"]
      selectFileName
    end
  end

  def selectFileName
    h = $conf.getConf
    if $fileDialog.showDialog(h["pathCurPix"]) !=0 then
      h["pathCurPix"]=$fileDialog.getSelectedFile
      $conf.setConf(h)
    end
  end

  def setFocus2Menu
    @mbar.setFocus
  end

  def execQr
    w = @tab.currentPageIndex
    case w
    when @idMail
      strData = @rqMail.getData
    when @idText
      strData = $codec.fromUnicode(@leStr.text).data.gsub(/\n/,"\r\n")
    end
    
    h=$conf.getConf
    if strData==@strLastData and h["sizeQr"]==@strLastSize and h["bMargin"]==@strMargin then
        $rqrDialog.exec
    elsif strData==@strLastData and (h["sizeQr"]!=@strLastSize or h["bMargin"]!=@strMargin) then
        $rqrCanvas.drawQr(@strLastQr)
        $rqrDialog.exec
    else
      setCaption(tr("RQR 変換中..."))
      @pCont = Proc.new {
        @strLastData=strData
        @strLastSize=h["sizeQr"]
        @strLastMargin=h["bMargin"]

        iLen = strData.size
        iVer = 0
        (1..@aiVMax.size-1).each do |i|
          if @aiVMax[i] > iLen then
            iVer = i
            break
          end
        end
        
        h=$conf.getConf
        
        fp = File.new(h["pathTmp"],"w")
        fp.print(strData)
        fp.close

        @strLastQr=`#{ $strBinQr + " -v" + iVer.to_s + " -m8 -eM < " + h["pathTmp"]}`

        File.delete(h["pathTmp"])

        $rqrCanvas.drawQr(@strLastQr)
        setCaption(tr("RQR"))
        $rqrDialog.exec
      }
      startTimer(0)
    end
  end

  def timerEvent(e)
    killTimers
    @pCont.call
  end
end

class RqrFileDialog < QDialog
  def initialize
    super(nil, "RqrFileDialog", true) # Modal Dialog
    setCaption(tr("保存ファイル名指定"))
    vb=QVBox.new(self)
    if $env.getDesktopWidth == 640 then
      vb.setFixedSize(570,300)
    else
      vb.setFixedSize(200,200)
    end

    @dir = "/home/samba/Main_Memory"
    
    @dir.sub!(/\/usr\/mnt.rom\/cf/,"/home/samba/CF_Card")
    @dir.sub!(/\/usr\/mnt.rom\/card/,"/home/samba/SD_Card")
    if FileTest.directory?("/home/zaurus/") then
      @dir.sub!(/\/home\/zaurus\/Documents/,"/home/samba/Main_Memory")
    else
      @dir.sub!(/\/home\/root\/Documents/,"/home/samba/Main_Memory")
    end
    @dir.chomp!()
    @fname = ""

    @list = QListView.new(vb)
    if $env.getDesktopWidth == 640 then
      @list.addColumn("File Name",250)
      @list.addColumn("Size",100)
      @list.addColumn("DateTime",200)
    else
      @list.addColumn("File Name",90)
      @list.addColumn("Size",30)
      @list.addColumn("DateTime",80)
    end

    @list.setColumnAlignment(0,1)
    @list.setColumnAlignment(1,2)
    @list.setColumnAlignment(2,2)

    @f = Array.new()
    
    @le = QLineEdit.new(vb)
    
    connect( @list, QSIGNAL("returnPressed(QListViewItem*)"), self, 'selectFile')
  end

  def showDialog(df)
    @dir = File.dirname(df)
    @fname = File.basename(df)
    Dir.chdir(@dir)
    @le.setText(tr(@fname))
    showFiles
    exec
  end

  def showFiles
    @list.clear
    getDir
    i = 0
    setCaption(@dir)
    while i < @f.size
      fo = File.stat(@f[i])
      fs = fo.size / 1024
      fss = if fo.directory? then
              ""
            else
              fs.to_s + "Kb "
            end
      ft = fo.mtime.to_s
      li = QListViewItem.new(@list,@f[i],fss,ft)
      @list.setSelected(li, true) if @f[i] == @fname
      i += 1
    end
#    @fname = ""
  end

  def selectFile(selectedItem)
    sf = selectedItem.text(0).to_str
    if FileTest.directory?(sf)
      Dir.chdir(sf)
      @dir =Dir.pwd
      @dir = "/home/samba" if @dir == "/home/zaurus"
      @dir = "/home/samba" if @dir == "/mnt"
      @dir = "/home/samba" if @dir == "/usr/mnt.rom"
      Dir.chdir(@dir)
#      @fname = "."
      showFiles
    else
      @fname = sf
      @le.setText(tr(@fname))
    end
  end

  def getDir
    @f.clear
    @f = Dir.entries("./")
    @f.sort!
  end
  
  def getSelectedFile
    @dir + "/" + @fname
  end
end

a = QPEApplication.new([$0] + ARGV)

$env  = RqrEnv.new

if $env.getDesktopWidth == 640 then
  $iBlock=8
  QPEApplication.setFont(QFont.new("lcfont",18))
else
  $iBlock=4
  QPEApplication.setFont(QFont.new("lcfont",10))
end

a.setDefaultCodec(QTextCodec.codecForName("UTF-8"))

if FileTest.directory?("/home/zaurus/") then
  $conf = RqrConf.new
else
  $conf = RqrConf.new("/home/root/Settings/rqr.conf")
end
$sizeDialog = RqrSetPixSizeDialog.new
$fileDialog = RqrFileDialog.new
$frmMain = FormMain.new
$rqrDialog = RqrDialog.new
$rqrCanvas = $rqrDialog.getCanvas
a.showMainWidget($frmMain)
a.exec
